﻿using SoapEndpoint.Service.Proxy.Models.Flows;
using SoapEndpoint.Service.Proxy.Models.Systeem;
using SoapEndpoint.Tester.Infrastructure;
using InventarisatieMessageType = SoapEndpoint.Service.Flows.Inventarisatie.v1.LogbookService.BerichtType;
using SbEnSkMessageType = SoapEndpoint.Service.Flows.SbEnSk.v1.LogbookService.BerichtType;
using SchuldbemiddelingMessageType = SoapEndpoint.Service.Systeem.v1.LogbookService.Berichttype;
using WaarborgfondsMessageType = SoapEndpoint.Service.Flows.Waarborgfonds.v1.LogbookService.BerichtType;
using BrEnHfMessageType = SoapEndpoint.Service.Flows.BrEnHf.v1.LogbookService.BerichtType;
using BewindvoeringMessageType = SoapEndpoint.Service.Flows.Bewindvoering.v1.LogbookService.BerichtType;
using PauzeSignaal3MessageType = SoapEndpoint.Service.Flows.PauzeSignaal3.v1.LogbookService.BerichtType;

namespace SoapEndpoint.Tester.Scenario;

public abstract class ScenarioBase
{
    public ScenarioManager Manager { get; private set; }
    public ScenarioResultGroup ResultGroup { get; set; }

    protected ScenarioResult Result => ResultGroup.Result;
    protected Conversation Conversation => Result.Conversation;

    public AdresboekOrganisatieModel Se => Manager.Se;
    public AdresboekOrganisatieModel Shv => Manager.Shv;

    public abstract string Name { get; }
    public virtual bool DisplayInList => true;

    public abstract FlowType FlowType { get; }
    public abstract Version FlowVersion { get; }

    public virtual AdresboekOrganisatieModel SeOrganization =>
        Manager.Organizations.SingleOrDefault(p => p.Referentienummer.ToString()
            .Equals(Program.Configuration["SeStandardTestOrganizationReference"],
                StringComparison.CurrentCultureIgnoreCase));

    public virtual AdresboekOrganisatieModel ShvOrganization =>
        Manager.Organizations.SingleOrDefault(p => p.Referentienummer.ToString()
            .Equals(Program.Configuration["ShvStandardTestOrganizationReference"],
                StringComparison.CurrentCultureIgnoreCase));

    protected abstract Task ExecuteAsync();

    protected void StartNewScenario(string name)
    {
        ResultGroup.AddResult(new ScenarioResult(this, name, FlowType));
    }

    public ScenarioBase Initialize(ScenarioManager manager)
    {
        Manager = manager;
        ResultGroup = new ScenarioResultGroup(this);
        return this;
    }

    public virtual CertificateItem CertificateForOrganization(AdresboekOrganisatieModel organization)
    {
        return Manager.Certificates.FirstOrDefault(c => c.Name == organization.Naam);
    }

    public async Task<ScenarioResultGroup> ExecuteScenarioAsync()
    {
        await InitializeAsync();

        Manager.LogAction($"Starting scenario {Name}.");

        try
        {
            await ExecuteAsync();
        }
        catch (Exception ex)
        {
            Result.SetError(ex.Message);
        }

        if (Result.Success)
        {
            Manager.LogAction("Scenario succeeded.");
        }
        else
        {
            Manager.LogAction($"Scenario failed. Error: {Result.Error}", false);
        }

        return ResultGroup;
    }

    protected virtual async Task InitializeAsync()
    {
        Manager.LogAction("Cleaning up message boxes");

        switch (FlowType)
        {
            case FlowType.Schuldbemiddeling:
                await Manager.ClearMessagesAsync<SchuldbemiddelingMessageType>();
                break;
            case FlowType.SbEnSk:
                await Manager.ClearMessagesAsync<SbEnSkMessageType>();
                break;
            case FlowType.Waarborgfonds:
                await Manager.ClearMessagesAsync<WaarborgfondsMessageType>();
                break;
            case FlowType.Inventarisatie:
                await Manager.ClearMessagesAsync<InventarisatieMessageType>();
                break;
            case FlowType.BrEnHf:
                await Manager.ClearMessagesAsync<BrEnHfMessageType>();
                break;
            case FlowType.Bewindvoering:
                await Manager.ClearMessagesAsync<BewindvoeringMessageType>();
                break;
            case FlowType.PauzeSignaal3:
                await Manager.ClearMessagesAsync<PauzeSignaal3MessageType>();
                break;
            default:
                throw new ArgumentOutOfRangeException();
        }
    }

    public override string ToString() => Name;
}